import java.awt.*;

        // Layout a number of buttons to illustrate the ipad
        // attributes of a component's constraints.
class GridBagIpadFrame extends Frame {
    public GridBagIpadFrame(){
        final int width = 150, height = 150;
        setSize(width,height);
        // Use a slightly larger than normal font.
        setFont(new Font("Serif",Font.BOLD,20));

        // Create the layout manager.
        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        setupComponents();
    }

        // Setup the required set of components in this container.
    protected void setupComponents(){
        // Create the constraints. Reuse this object with each button.
        GridBagConstraints constraints = new GridBagConstraints();

        // Create each button, set up its constraints and
        // add it to the frame.
        final int singleCell = 1;
        // Keep track of the default padding
        int defaultXpad = constraints.ipadx;
        int defaultYpad = constraints.ipady;

        initBasicConstraints(constraints,0,0,singleCell,singleCell);
        addButton("1",constraints);

        initBasicConstraints(constraints,1,0,singleCell,singleCell);
        // Define a small amount of marginal padding for this button.
        final int padding = 20;
        constraints.ipadx = padding;
        constraints.ipady = padding;
        addButton("2",constraints);
        // Restore the default.
        constraints.ipadx = defaultXpad;
        constraints.ipady = defaultYpad;

        initBasicConstraints(constraints,0,1,singleCell,singleCell);
        addButton("3",constraints);

        initBasicConstraints(constraints,1,1,singleCell,singleCell);
        addButton("4",constraints);
    }

        // Initialise the x,y width,height values for the constraints.
        // Each component has an equal non-zero weight.
    protected void initBasicConstraints(GridBagConstraints constraints,
                                int x, int y, int width, int height){
        constraints.gridx = x;
        constraints.gridy = y;
        constraints.gridwidth = width;
        constraints.gridheight = height;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.fill = GridBagConstraints.NONE;
    }

        // Add a button to the Frame with the given label and constraints.
    protected void addButton(String label, GridBagConstraints constraints){
        Button b = new Button(label);
        add(b,constraints);
    }
}
