import java.net.*;
import java.io.*;

        // A server that accepts multiple client connections.
        // The service, implemented by the RandomService class,
        // sends random integers when requested.
class RandomServiceListener {
    public RandomServiceListener(int port) throws Exception {
        try{
            // Listen on port.
            serverSocket = new ServerSocket(port);
        }
        catch(BindException e){
            throw new Exception("Failed to create a server socket: "+
                        e.getMessage());
        }
    }

        // Listen for clients. When each arrives, hand the socket
        // over to a RandomService slave.
    public void listen() throws Exception {
        ServerSocket serverSocket = getServerSocket();
        // Wait up to 30 minutes for new clients.
        final int timeToWait = 1*60*1000;
        boolean keepWaiting = true;

        while(keepWaiting){
            try{
                // Wait for a client to make contact.
                serverSocket.setSoTimeout(timeToWait);
                Socket clientSocket = serverSocket.accept();
                // Let a separate Thread handle it.
                new Thread(new RandomService(clientSocket)).start();
            }
            catch(InterruptedIOException e){
                // We timed out waiting for a client.
                keepWaiting = false;
            }
            catch(IOException e){
                throw new Exception("IOException: "+ e.getMessage());
            }
        }
        // No more clients, so close the socket.
        serverSocket.close();
    }

    protected ServerSocket getServerSocket(){
        return serverSocket;
    }


    // The socket on which the listening is done.
    private final ServerSocket serverSocket;
}
