import java.io.*;
import java.net.*;

        // Create a URL from the given String.
        // Fetch its contents on request.
class URLHandler {
    public URLHandler(String url) throws Exception {
        resource = new URL(url);
    }

        // Fetch the resource. Save it in a local file that matches
        // the suffix of the resource. Use index.html if no suffix
        // is found.
    public void fetch() throws Exception {
        URL resource = getResource();
        // Use the resource file name as the basis of a local
        // name in which to save it.
        String filename = resource.getFile();
        // Provide a default name.
        String localName = "index.html";

        // Find the last part of the resource name.
        int lastPart = filename.lastIndexOf('/');
        if(lastPart >= 0){
            // Skip the final slash.
            lastPart++;
            // Make sure there is something there.
            if(lastPart < filename.length()){
                localName = filename.substring(lastPart);
            }
        }

        System.out.println("Saving "+filename+" as "+localName);

        copyFile(resource.openStream(),localName);
    }

        // Copy the input stream to the file named in localName.
    protected void copyFile(InputStream inputStream,String localName)
                                                throws IOException {
        OutputStream outputStream = new FileOutputStream(localName);
        // Create the buffer for the transfer.
        final int buffLen = 1024;
        byte[] buffer = new byte[buffLen];
        int bytesRead = inputStream.read(buffer);
        while(bytesRead > 0){
            outputStream.write(buffer,0,bytesRead);
            bytesRead = inputStream.read(buffer);
        }
        inputStream.close();
        outputStream.close();
    }

    protected URL getResource(){
        return resource;
    }

    private final URL resource;
}
