import javax.swing.*;
import java.util.*;

        // Track an individual ship. This receives
        // notifications each time its ship moves.
        // When notified, it should tell its GUI to
        // repaint itself.
class ShipTracker implements Observer {
    public ShipTracker(Ship ship, JComponent gui){
        this.gui = gui;
        setShip(ship);
        // Initialise the position attributes.
        updatePath(ship.getCourseAndPosition());
    }

        // Provide access to the path of the ship.
        // Strictly speaking, this should be read-only
        // access, but that prevents synchronization.
    public List getShipTrack(){
        // Make sure the track is up to date.
        updatePath(ship.getCourseAndPosition());
        return getTrack();
    }

        // Observer interface - a Ship has moved.
    public void update(Observable o, Object arg){
        updatePath((CourseAndPosition) arg);
        getGui().repaint();
    }

    public void updatePath(CourseAndPosition position){
        CourseAndPosition lastPosition = null;
        List track = getTrack();
        int numberOfPositions = track.size();
        if(numberOfPositions != 0){
             lastPosition = (CourseAndPosition) track.get(numberOfPositions-1);
        }
        // Don't add repeated positions.
        if(!position.equals(lastPosition)){
            track.add(position);
        }
    }

    public Ship getShip(){
        return ship;
    }

    public void setShip(Ship s){
        // Stop observing any previous ship.
        if(ship != null){
            ship.deleteObserver(this);
            // Clear the track.
            getTrack().clear();
        }
        ship = s;
        ship.addObserver(this);
    }

    protected JComponent getGui(){
        return gui;
    }

    protected List getTrack(){
        return track;
    }

    // The Ship being tracked.
    private Ship ship;
    // Where have we been. Access to the list must be synchronized.
    private List track = Collections.synchronizedList(new LinkedList());
    // Where the track is drawn.
    private final JComponent gui;
}
